@@ -73,7 +73,7 @@ impl Operator {
             .unwrap_or_default();
 
         Ok(Operator {
-            core: build_operator(scheme.clone(), map.clone())?.blocking(),
+            core: build_operator(scheme, map.clone())?.blocking(),
             __scheme: scheme,
             __map: map,
         })
@@ -84,7 +84,7 @@ impl Operator {
         let op = layer.0.layer(self.core.clone().into());
         Ok(Self {
             core: op.blocking(),
-            __scheme: self.__scheme.clone(),
+            __scheme: self.__scheme,
             __map: self.__map.clone(),
         })
     }
@@ -121,7 +121,7 @@ impl Operator {
     #[pyo3(signature = (path, bs, **kwargs))]
     pub fn write(&self, path: PathBuf, bs: Vec<u8>, kwargs: Option<WriteOptions>) -> PyResult<()> {
         let path = path.to_string_lossy().to_string();
-        let kwargs = kwargs.unwrap_or_default();
+        let mut kwargs = kwargs.unwrap_or_default();
         let mut write = self
             .core
             .write_with(&path, bs)
@@ -138,6 +138,9 @@ impl Operator {
         if let Some(cache_control) = &kwargs.cache_control {
             write = write.cache_control(cache_control);
         }
+        if let Some(user_metadata) = kwargs.user_metadata.take() {
+            write = write.user_metadata(user_metadata);
+        }
 
         write.call().map(|_| ()).map_err(format_pyerr)
     }
@@ -236,7 +239,7 @@ impl Operator {
     pub fn to_async_operator(&self) -> PyResult<AsyncOperator> {
         Ok(AsyncOperator {
             core: self.core.clone().into(),
-            __scheme: self.__scheme.clone(),
+            __scheme: self.__scheme,
             __map: self.__map.clone(),
         })
     }
@@ -259,7 +262,7 @@ impl Operator {
         let args = vec![self.__scheme.to_string()];
         let args = PyTuple::new(py, args)?.into_py_any(py)?;
         let kwargs = self.__map.clone().into_py_any(py)?;
-        Ok(PyTuple::new(py, [args, kwargs])?.into_py_any(py)?)
+        PyTuple::new(py, [args, kwargs])?.into_py_any(py)
     }
 }
 
@@ -292,7 +295,7 @@ impl AsyncOperator {
             .unwrap_or_default();
 
         Ok(AsyncOperator {
-            core: build_operator(scheme.clone(), map.clone())?.into(),
+            core: build_operator(scheme, map.clone())?,
             __scheme: scheme,
             __map: map,
         })
@@ -303,7 +306,7 @@ impl AsyncOperator {
         let op = layer.0.layer(self.core.clone());
         Ok(Self {
             core: op,
-            __scheme: self.__scheme.clone(),
+            __scheme: self.__scheme,
             __map: self.__map.clone(),
         })
     }
@@ -358,7 +361,7 @@ impl AsyncOperator {
         bs: &Bound<PyBytes>,
         kwargs: Option<WriteOptions>,
     ) -> PyResult<Bound<'p, PyAny>> {
-        let kwargs = kwargs.unwrap_or_default();
+        let mut kwargs = kwargs.unwrap_or_default();
         let this = self.core.clone();
         let bs = bs.as_bytes().to_vec();
         let path = path.to_string_lossy().to_string();
@@ -378,6 +381,10 @@ impl AsyncOperator {
             if let Some(cache_control) = &kwargs.cache_control {
                 write = write.cache_control(cache_control);
             }
+            if let Some(user_metadata) = kwargs.user_metadata.take() {
+                write = write.user_metadata(user_metadata);
+            }
+
             write.await.map(|_| ()).map_err(format_pyerr)
         })
     }
@@ -572,6 +579,26 @@ impl AsyncOperator {
         })
     }
 
+    /// Presign an operation for delete which expires after `expire_second` seconds.
+    pub fn presign_delete<'p>(
+        &'p self,
+        py: Python<'p>,
+        path: PathBuf,
+        expire_second: u64,
+    ) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
+        future_into_py(py, async move {
+            let res = this
+                .presign_delete(&path, Duration::from_secs(expire_second))
+                .await
+                .map_err(format_pyerr)
+                .map(PresignedRequest)?;
+
+            Ok(res)
+        })
+    }
+
     pub fn capability(&self) -> PyResult<capability::Capability> {
         Ok(capability::Capability::new(
             self.core.info().full_capability(),
@@ -581,7 +608,7 @@ impl AsyncOperator {
     pub fn to_operator(&self) -> PyResult<Operator> {
         Ok(Operator {
             core: self.core.clone().blocking(),
-            __scheme: self.__scheme.clone(),
+            __scheme: self.__scheme,
             __map: self.__map.clone(),
         })
     }
@@ -608,7 +635,7 @@ impl AsyncOperator {
         let args = vec![self.__scheme.to_string()];
         let args = PyTuple::new(py, args)?.into_py_any(py)?;
         let kwargs = self.__map.clone().into_py_any(py)?;
-        Ok(PyTuple::new(py, [args, kwargs])?.into_py_any(py)?)
+        PyTuple::new(py, [args, kwargs])?.into_py_any(py)
     }
 }
 
