@@ -15,17 +15,17 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::collections::VecDeque;
-use std::fmt;
-use std::fmt::Debug;
-use std::fmt::Formatter;
-
 use bytes::Bytes;
 use http::header;
 use http::Request;
 use http::Response;
 use http::StatusCode;
 use serde::Deserialize;
+use std::collections::VecDeque;
+use std::fmt;
+use std::fmt::Debug;
+use std::fmt::Formatter;
+use std::sync::Arc;
 
 use super::error::parse_error;
 use crate::raw::*;
@@ -69,13 +69,11 @@ static HEADER_DESTINATION: &str = "Destination";
 static HEADER_OVERWRITE: &str = "Overwrite";
 
 pub struct WebdavCore {
+    pub info: Arc<AccessorInfo>,
     pub endpoint: String,
     pub server_path: String,
     pub root: String,
-    pub disable_copy: bool,
     pub authorization: Option<String>,
-
-    pub client: HttpClient,
 }
 
 impl Debug for WebdavCore {
@@ -111,9 +109,9 @@ impl WebdavCore {
             .body(Buffer::from(Bytes::from(PROPFIND_REQUEST)))
             .map_err(new_request_build_error)?;
 
-        let resp = self.client.send(req).await?;
+        let resp = self.info.http_client().send(req).await?;
         if !resp.status().is_success() {
-            return Err(parse_error(resp).await?);
+            return Err(parse_error(resp));
         }
 
         let bs = resp.into_body();
@@ -151,7 +149,7 @@ impl WebdavCore {
 
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
-        self.client.fetch(req).await
+        self.info.http_client().fetch(req).await
     }
 
     pub async fn webdav_put(
@@ -184,7 +182,7 @@ impl WebdavCore {
 
         let req = req.body(body).map_err(new_request_build_error)?;
 
-        self.client.send(req).await
+        self.info.http_client().send(req).await
     }
 
     pub async fn webdav_delete(&self, path: &str) -> Result<Response<Buffer>> {
@@ -199,7 +197,7 @@ impl WebdavCore {
 
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
-        self.client.send(req).await
+        self.info.http_client().send(req).await
     }
 
     pub async fn webdav_copy(&self, from: &str, to: &str) -> Result<Response<Buffer>> {
@@ -225,7 +223,7 @@ impl WebdavCore {
 
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
-        self.client.send(req).await
+        self.info.http_client().send(req).await
     }
 
     pub async fn webdav_move(&self, from: &str, to: &str) -> Result<Response<Buffer>> {
@@ -251,7 +249,7 @@ impl WebdavCore {
 
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
-        self.client.send(req).await
+        self.info.http_client().send(req).await
     }
 
     pub async fn webdav_list(&self, path: &str, args: &OpList) -> Result<Response<Buffer>> {
@@ -276,7 +274,7 @@ impl WebdavCore {
             .body(Buffer::from(Bytes::from(PROPFIND_REQUEST)))
             .map_err(new_request_build_error)?;
 
-        self.client.send(req).await
+        self.info.http_client().send(req).await
     }
 
     /// Create dir recursively for given path.
@@ -292,7 +290,7 @@ impl WebdavCore {
 
         loop {
             match self.webdav_stat_rooted_abs_path(path).await {
-                // Dir is exist, break the loop.
+                // Dir exists, break the loop.
                 Ok(_) => {
                     break;
                 }
@@ -332,7 +330,7 @@ impl WebdavCore {
 
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
-        let resp = self.client.send(req).await?;
+        let resp = self.info.http_client().send(req).await?;
         let status = resp.status();
 
         match status {
@@ -346,7 +344,7 @@ impl WebdavCore {
 
                 Ok(())
             }
-            _ => Err(parse_error(resp).await?),
+            _ => Err(parse_error(resp)),
         }
     }
 }
@@ -412,7 +410,7 @@ pub fn parse_propstat(propstat: &Propstat) -> Result<Metadata> {
     m.set_last_modified(parse_datetime_from_rfc2822(getlastmodified)?);
 
     // the storage services have returned all the properties
-    Ok(m.with_metakey(Metakey::Complete))
+    Ok(m)
 }
 
 #[derive(Deserialize, Debug, PartialEq, Eq, Clone, Default)]
